/**================================================================================================
Hochschule Luzern - Technik & Architektur                                  Mikrocontroller - HS2012
Copyright 2012 Felix Rohrer                                                    http://hslu.ximit.ch
---------------------------------------------------------------------------------------------------
Name        : Uebung_4-1.c
Author      : Felix Rohrer <felix.rohrer@stud.hslu.ch>
Version     : 2012-10-07v1.0
Description : Taschenrechner polnische Notation:
              o Verwenden Sie das Template main.c aus dem Anhang als Vorlage
              o "stdin" ist hier das File "inputfile1.txt" in dem Pfad von main (File mit
			    NetBeans als Editor verndern  Win-Text-Editor macht kein EOF wie gewnscht)
              o Sie knnen auch ber die Konsole die Eingabe machen  EOF mittels ctrl+d
              o Alle Funktionen sind zuerst in einem Source-File (main... File) zu halten
              o Zeitbedarf 45': codieren, debugging und testen
=================================================================================================*/

#include <stdio.h>
#include <stdlib.h>		/* fuer atof() */
#include <ctype.h>		/* fuer isdigit() */

/* Konstanten */
#define MAXOP 100		/* max Laenge von Operand und/oder Operator */
#define BUFSIZE 100		/* Buffer Size */
#define MAXVAL 100		/* Stack Size */
#define NUMBER '0'		/* Anzeige, Zahl */

/* Globale Variablen */
char buf[BUFSIZE];		/* Buffer */
int bufp = 0;			/* Buffer-Pointer */
double val[MAXVAL];		/* Stack */
int sp = 0;				/* Stack-Pointer */

/* Funktions-Deklaration */
void push(double f);	/* push: f auf den Stack bringen */
double pop(void);		/* pop: Wert von Stack holen und liefern */
int getop(char s[]);	/* getop: naechsten Operator und /oder numerischen Operanden holen */
int getch(void);		/* getch: Zeichen holen mit Buffer */
void ungetch(int c);	/* ungetch: Zeichen wieder zurueckstellen wenn zuviel geholt */



/**
 * The main procedure to run the application.
 * @param argc Number of command line arguments
 * @param argv The forwarded command line arguments
 * @return Application return (error) code
 */
int main(int argc, char** argv)
{
	/* Taschenrechner in umgekehrter polnischer Notation - Beispiel S. 74 K&R */
	
	int type;
	double op2;
	char s[MAXOP];

	//Oeffnet das File "inputfile.txt" in dem Pfad von main.c und nimmt das File als
	//Standard Eingabe anstelle Tastatur
	freopen("inputfile1.txt", "r", stdin);
	while ((type = getop(s)) != EOF) {
		switch (type) {
		case NUMBER:	// Zahl auf Stack pushen
			push(atof(s));
			break;
		case '+':		// + Plus
			push(pop() + pop());
			break;
		case '-':		// - Minus
			op2 = pop();
			push(pop() - op2);
			break;
		case '*':		// * Multiplikation
			push(pop() * pop());
			break;
		case '/':		// / Division
			op2 = pop();
			if (op2 != 0.0) {
				push(pop() / op2);
			} else {
				printf("Error: Division durch 0!\n");
			}
			break;
		case '\n':		// Resultat Ausgeben
			printf("Result:\t%.8g\n", pop());
			break;
		default:		// Unbekannter Befehl
				printf("Error: Unbekannter Befehle: %s\n", s);
				break;
		} // switch (type)
	} // while ( != EOF)
	
	fclose(stdin); // Schliesst die Standard-Eingabe stdin (das File von freopen)

	return(EXIT_SUCCESS);
}


/* push: f auf den Stack bringen */
void push(double f) {
	if (sp < MAXVAL) {
		val[sp++] = f;
	} else {
		printf("Error: Stack ist voll! %g kann nicht eingefuegt werden...\n", f);
	}
}

/*pop: Wert von Stack holen und liefern */
double pop(void) {
	if (sp > 0) {
		return val[--sp];
	} else {
		printf("Error: Stack ist leer!\n");
		return 0.0;
	}
}

/*getop: naechsten Operator und /oder numerischen Operanden holen */
int getop(char s[]){
    int i, c;

    while ((s[0] = c = getch()) == ' ' || c == '\t')
		; // whitespaces entfernen - tue nichts bei diese Zeichen

    s[1] = '\0'; // String terminieren
    if (!isdigit(c) && c != '.') {
		return c; /* keine Zahl --> Befehl */
	}
    i = 0;
    if (isdigit(c)) { /* ganzzahligen Teil sammeln */
        while (isdigit(s[++i] = c = getch())) // solange es eine Zahl ist
            ;
	}
    if (c == '.') { /* Dezimalstellen */
        while (isdigit(s[++i] = c = getch())) // solange es eine Zahl ist
            ;
	}
    s[i] = '\0'; // String terminieren
    if (c != EOF) { // solange nicht am Ende von der Zeile
        ungetch(c); // Zeichen zurck auf Stack
	}

    return NUMBER; // default return wert 0
}

/* getch: Zeichen holen mit Buffer */
int getch(void) {
	// return (bufp > 0) ? buf[--bufp] : getchar();
	if (bufp > 0) {
		return buf[--bufp];
	} else {
		return getchar();
	}
}

/* ungetch: Zeichen wieder zurueckstellen wenn zuviel geholt */
void ungetch(int c){
	if (bufp >= BUFSIZE) {
        printf("Error ungetch(): Zu viele Zeichen\n");
	} else {
        buf[bufp++] = c;
	}
}
